@@ -1,14 +1,13 @@
 # -*- coding: utf-8 -*-
 
 # Import Python Libs
-from __future__ import absolute_import
-from __future__ import print_function
-import json
+from __future__ import absolute_import, print_function, unicode_literals
 import time
 import threading
 
 # Import Salt Libs
-import salt.utils
+import salt.utils.json
+import salt.utils.stringutils
 from salt.netapi.rest_tornado import saltnado
 from salt.utils.versions import StrictVersion
 
@@ -18,19 +17,9 @@ from tests.support.helpers import flaky
 from tests.support.unit import skipIf
 
 # Import 3rd-party libs
-import salt.ext.six as six
-try:
-    import zmq
-    from zmq.eventloop.ioloop import ZMQIOLoop
-    HAS_ZMQ_IOLOOP = True
-except ImportError:
-    HAS_ZMQ_IOLOOP = False
-
-
-def json_loads(data):
-    if six.PY3 and isinstance(data, bytes):
-        data = data.decode('utf-8')
-    return json.loads(data)
+from salt.ext import six
+from salt.utils.zeromq import zmq, ZMQDefaultLoop as ZMQIOLoop
+HAS_ZMQ_IOLOOP = bool(zmq)
 
 
 class _SaltnadoIntegrationTestCase(SaltnadoTestCase):  # pylint: disable=abstract-method
@@ -53,6 +42,7 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
         application = self.build_tornado_app(urls)
 
         application.event_listener = saltnado.EventListener({}, self.opts)
+        self.application = application
         return application
 
     def test_root(self):
@@ -64,7 +54,7 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
                               request_timeout=30,
                 )
         self.assertEqual(response.code, 200)
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(sorted(response_obj['clients']),
                          ['local', 'local_async', 'runner', 'runner_async'])
         self.assertEqual(response_obj['return'], 'Welcome')
@@ -80,7 +70,7 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json']},
                               follow_redirects=False,
                               connect_timeout=30,
@@ -90,6 +80,7 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
         self.assertEqual(response.headers['Location'], '/login')
 
     # Local client tests
+
     @skipIf(True, 'to be re-enabled when #23623 is merged')
     def test_simple_local_post(self):
         '''
@@ -101,14 +92,15 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               connect_timeout=30,
                               request_timeout=30,
                               )
-        response_obj = json_loads(response.body)
-        self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
+        response_obj = salt.utils.json.loads(response.body)
+        self.assertEqual(len(response_obj['return']), 1)
+        self.assertEqual(response_obj['return'][0], {'minion': True, 'sub_minion': True})
 
     def test_simple_local_post_no_tgt(self):
         '''
@@ -120,16 +112,17 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               connect_timeout=30,
                               request_timeout=30,
                               )
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(response_obj['return'], ["No minions matched the target. No command was sent, no jid was assigned."])
 
     # local client request body test
+
     @skipIf(True, 'Undetermined race condition in test. Temporarily disabled.')
     def test_simple_local_post_only_dictionary_request(self):
         '''
@@ -141,14 +134,15 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
               }
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               connect_timeout=30,
                               request_timeout=30,
                               )
-        response_obj = json_loads(response.body)
-        self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
+        response_obj = salt.utils.json.loads(response.body)
+        self.assertEqual(len(response_obj['return']), 1)
+        self.assertEqual(response_obj['return'][0], {'minion': True, 'sub_minion': True})
 
     def test_simple_local_post_invalid_request(self):
         '''
@@ -157,7 +151,7 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
         low = ["invalid request"]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               connect_timeout=30,
@@ -173,12 +167,12 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
 
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
 
@@ -198,12 +192,12 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
 
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
         ret[1]['minions'] = sorted(ret[1]['minions'])
@@ -232,12 +226,12 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 ]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
 
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
         ret[1]['minions'] = sorted(ret[1]['minions'])
@@ -256,13 +250,40 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(response_obj['return'], [{}])
 
+    @skipIf(True, 'Undetermined race condition in test. Temporarily disabled.')
+    def test_simple_local_post_only_dictionary_request_with_order_masters(self):
+        '''
+        Test a basic API of /
+        '''
+        low = {'client': 'local',
+                'tgt': '*',
+                'fun': 'test.ping',
+              }
+
+        self.application.opts['order_masters'] = True
+        self.application.opts['syndic_wait'] = 5
+
+        response = self.fetch('/',
+                              method='POST',
+                              body=salt.utils.json.dumps(low),
+                              headers={'Content-Type': self.content_type_map['json'],
+                                       saltnado.AUTH_TOKEN_HEADER: self.token['token']},
+                              connect_timeout=30,
+                              request_timeout=30,
+                              )
+        response_obj = salt.utils.json.loads(response.body)
+
+        self.application.opts['order_masters'] = []
+        self.application.opts['syndic_wait'] = 5
+        self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
+
     # runner tests
     def test_simple_local_runner_post(self):
         low = [{'client': 'runner',
@@ -270,13 +291,13 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               connect_timeout=30,
                               request_timeout=30,
                               )
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(len(response_obj['return']), 1)
         self.assertEqual(set(response_obj['return'][0]), set(['minion', 'sub_minion']))
 
@@ -287,13 +308,13 @@ class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               connect_timeout=10,
                               request_timeout=10,
                               )
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertIn('return', response_obj)
         self.assertEqual(1, len(response_obj['return']))
         self.assertIn('jid', response_obj['return'][0])
@@ -318,7 +339,7 @@ class TestMinionSaltAPIHandler(_SaltnadoIntegrationTestCase):
                               headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               follow_redirects=False,
                               )
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(len(response_obj['return']), 1)
         # one per minion
         self.assertEqual(len(response_obj['return'][0]), 2)
@@ -333,24 +354,24 @@ class TestMinionSaltAPIHandler(_SaltnadoIntegrationTestCase):
                               headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               follow_redirects=False,
                               )
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(len(response_obj['return']), 1)
         self.assertEqual(len(response_obj['return'][0]), 1)
         # check a single grain
         self.assertEqual(response_obj['return'][0]['minion']['id'], 'minion')
 
     def test_post(self):
-        low = [{'tgt': '*',
+        low = [{'tgt': '*minion',
                 'fun': 'test.ping',
                 }]
         response = self.fetch('/minions',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
 
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
 
@@ -362,17 +383,17 @@ class TestMinionSaltAPIHandler(_SaltnadoIntegrationTestCase):
     def test_post_with_client(self):
         # get a token for this test
         low = [{'client': 'local_async',
-                'tgt': '*',
+                'tgt': '*minion',
                 'fun': 'test.ping',
                 }]
         response = self.fetch('/minions',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
 
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
 
@@ -383,7 +404,7 @@ class TestMinionSaltAPIHandler(_SaltnadoIntegrationTestCase):
 
     def test_post_with_incorrect_client(self):
         '''
-        The /minions endpoint is async only, so if you try something else
+        The /minions endpoint is asynchronous only, so if you try something else
         make sure you get an error
         '''
         # get a token for this test
@@ -393,7 +414,7 @@ class TestMinionSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 }]
         response = self.fetch('/minions',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
@@ -420,7 +441,7 @@ class TestJobsSaltAPIHandler(_SaltnadoIntegrationTestCase):
                                follow_redirects=False,
                                )
         response = self.wait(timeout=30)
-        response_obj = json_loads(response.body)['return'][0]
+        response_obj = salt.utils.json.loads(response.body)['return'][0]
         try:
             for jid, ret in six.iteritems(response_obj):
                 self.assertIn('Function', ret)
@@ -430,7 +451,7 @@ class TestJobsSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 self.assertIn('StartTime', ret)
                 self.assertIn('Arguments', ret)
         except AttributeError as attribute_error:
-            print(json_loads(response.body))
+            print(salt.utils.json.loads(response.body))
             raise
 
         # test with a specific JID passed in
@@ -442,7 +463,7 @@ class TestJobsSaltAPIHandler(_SaltnadoIntegrationTestCase):
                                follow_redirects=False,
                                )
         response = self.wait(timeout=30)
-        response_obj = json_loads(response.body)['return'][0]
+        response_obj = salt.utils.json.loads(response.body)['return'][0]
         self.assertIn('Function', response_obj)
         self.assertIn('Target', response_obj)
         self.assertIn('Target-type', response_obj)
@@ -471,11 +492,11 @@ class TestRunSaltAPIHandler(_SaltnadoIntegrationTestCase):
                 }]
         response = self.fetch('/run',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
-        response_obj = json_loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
 
 
@@ -562,7 +583,7 @@ class TestWebhookSaltAPIHandler(_SaltnadoIntegrationTestCase):
                                   body='foo=bar',
                                   headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                                   )
-            response_obj = json_loads(response.body)
+            response_obj = salt.utils.json.loads(response.body)
             self.assertTrue(response_obj['success'])
             resolve_future_timeout = 60
             self._future_resolved.wait(resolve_future_timeout)
@@ -574,7 +595,7 @@ class TestWebhookSaltAPIHandler(_SaltnadoIntegrationTestCase):
             self.assertIn('headers', event['data'])
             self.assertEqual(
                 event['data']['post'],
-                {'foo': salt.utils.to_bytes('bar')}
+                {'foo': salt.utils.stringutils.to_bytes('bar')}
             )
         finally:
             self._future_resolved.clear()
