@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
+    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
 
 
     ====================================
@@ -13,11 +13,10 @@
 # pylint: disable=repr-flag-used-in-string
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 import os
 import re
 import sys
-import json
 import time
 import stat
 import errno
@@ -30,13 +29,15 @@ from datetime import datetime, timedelta
 
 # Import salt testing libs
 from tests.support.unit import TestCase
-from tests.support.helpers import RedirectStdStreams, requires_sshd_server
+from tests.support.helpers import (
+    RedirectStdStreams, requires_sshd_server, win32_kill_process_tree
+)
 from tests.support.runtests import RUNTIME_VARS
 from tests.support.mixins import AdaptedConfigurationTestCaseMixin, SaltClientTestCaseMixin
 from tests.support.paths import ScriptPathMixin, INTEGRATION_TEST_DIR, CODE_DIR, PYEXEC, SCRIPT_DIR
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 from salt.ext.six.moves import cStringIO  # pylint: disable=import-error
 
 STATE_FUNCTION_RUNNING_RE = re.compile(
@@ -84,9 +85,9 @@ class ShellTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
             log.debug('Generating {0}'.format(script_path))
 
             # Late import
-            import salt.utils
+            import salt.utils.files
 
-            with salt.utils.fopen(script_path, 'w') as sfh:
+            with salt.utils.files.fopen(script_path, 'w') as sfh:
                 script_template = SCRIPT_TEMPLATES.get(script_name, None)
                 if script_template is None:
                     script_template = SCRIPT_TEMPLATES.get('common', None)
@@ -97,10 +98,15 @@ class ShellTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                             script_name
                         )
                     )
-                sfh.write(
+                contents = (
                     '#!{0}\n'.format(sys.executable) +
                     '\n'.join(script_template).format(script_name.replace('salt-', ''))
                 )
+                sfh.write(contents)
+                log.debug(
+                    'Wrote the following contents to temp script %s:\n%s',
+                    script_path, contents
+                )
             st = os.stat(script_path)
             os.chmod(script_path, st.st_mode | stat.S_IEXEC)
 
@@ -121,8 +127,8 @@ class ShellTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                     data = '\n'.join(data)
                     self.assertIn('minion', data)
         '''
-        arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
-        return self.run_script('salt', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr)
+        arg_str = '-c {0} -t {1} {2}'.format(self.get_config_dir(), timeout, arg_str)
+        return self.run_script('salt', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr, timeout=timeout)
 
     def run_ssh(self, arg_str, with_retcode=False, timeout=25,
                 catch_stderr=False, wipe=False, raw=False):
@@ -139,15 +145,28 @@ class ShellTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
         )
         return self.run_script('salt-ssh', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr, raw=True)
 
-    def run_run(self, arg_str, with_retcode=False, catch_stderr=False, async=False, timeout=60, config_dir=None):
+    def run_run(self,
+                arg_str,
+                with_retcode=False,
+                catch_stderr=False,
+                asynchronous=False,
+                timeout=60,
+                config_dir=None,
+                **kwargs):
         '''
         Execute salt-run
         '''
-        arg_str = '-c {0}{async_flag} -t {timeout} {1}'.format(config_dir or self.get_config_dir(),
-                                                 arg_str,
-                                                 timeout=timeout,
-                                                 async_flag=' --async' if async else '')
-        return self.run_script('salt-run', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr)
+        asynchronous = kwargs.get('async', asynchronous)
+        arg_str = '-c {0}{async_flag} -t {timeout} {1}'.format(
+                config_dir or self.get_config_dir(),
+                arg_str,
+                timeout=timeout,
+                async_flag=' --async' if asynchronous else '')
+        return self.run_script('salt-run',
+                               arg_str,
+                               with_retcode=with_retcode,
+                               catch_stderr=catch_stderr,
+                               timeout=timeout)
 
     def run_run_plus(self, fun, *arg, **kwargs):
         '''
@@ -228,11 +247,18 @@ class ShellTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                    arg_str,
                    catch_stderr=False,
                    with_retcode=False,
+                   catch_timeout=False,
                    # FIXME A timeout of zero or disabling timeouts may not return results!
                    timeout=15,
-                   raw=False):
+                   raw=False,
+                   log_output=None):
         '''
         Execute a script with the given argument string
+
+        The ``log_output`` argument is ternary, it can be True, False, or None.
+        If the value is boolean, then it forces the results to either be logged
+        or not logged. If it is None, then the return code of the subprocess
+        determines whether or not to log results.
         '''
         script_path = self.get_script_path(script)
         if not os.path.isfile(script_path):
@@ -276,51 +302,99 @@ class ShellTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
 
             popen_kwargs['preexec_fn'] = detach_from_parent_group
 
-        elif sys.platform.lower().startswith('win') and timeout is not None:
-            raise RuntimeError('Timeout is not supported under windows')
+        def format_return(retcode, stdout, stderr=None, timed_out=False):
+            '''
+            DRY helper to log script result if it failed, and then return the
+            desired output based on whether or not stderr was desired, and
+            wither or not a retcode was desired.
+            '''
+            log_func = log.debug
+            if timed_out:
+                log.error(
+                    'run_script timed out after %d seconds (process killed)',
+                    timeout
+                )
+                log_func = log.error
+
+            if log_output is True \
+                    or timed_out \
+                    or (log_output is None and retcode != 0):
+                log_func(
+                    'run_script results for: %s %s\n'
+                    'return code: %s\n'
+                    'stdout:\n'
+                    '%s\n\n'
+                    'stderr:\n'
+                    '%s',
+                    script, arg_str, retcode, stdout, stderr
+                )
+
+            stdout = stdout or ''
+            stderr = stderr or ''
+
+            if not raw:
+                stdout = stdout.splitlines()
+                stderr = stderr.splitlines()
+
+            ret = [stdout]
+            if catch_stderr:
+                ret.append(stderr)
+            if with_retcode:
+                ret.append(retcode)
+            if catch_timeout:
+                ret.append(timed_out)
+
+            return ret[0] if len(ret) == 1 else tuple(ret)
 
         process = subprocess.Popen(cmd, **popen_kwargs)
 
+        # Late import
+        import salt.utils.platform
+
         if timeout is not None:
             stop_at = datetime.now() + timedelta(seconds=timeout)
             term_sent = False
             while True:
                 process.poll()
                 time.sleep(0.1)
-                if datetime.now() > stop_at:
+                if datetime.now() <= stop_at:
+                    # We haven't reached the timeout yet
+                    if process.returncode is not None:
+                        break
+                else:
+                    # We've reached the timeout
                     if term_sent is False:
                         # Kill the process group since sending the term signal
                         # would only terminate the shell, not the command
                         # executed in the shell
-                        os.killpg(os.getpgid(process.pid), signal.SIGINT)
+                        if salt.utils.platform.is_windows():
+                            _, alive = win32_kill_process_tree(process.pid)
+                            if alive:
+                                log.error("Child processes still alive: %s", alive)
+                        else:
+                            os.killpg(os.getpgid(process.pid), signal.SIGINT)
                         term_sent = True
                         continue
 
                     try:
                         # As a last resort, kill the process group
-                        os.killpg(os.getpgid(process.pid), signal.SIGKILL)
+                        if salt.utils.platform.is_windows():
+                            _, alive = win32_kill_process_tree(process.pid)
+                            if alive:
+                                log.error("Child processes still alive: %s", alive)
+                        else:
+                            os.killpg(os.getpgid(process.pid), signal.SIGINT)
                     except OSError as exc:
                         if exc.errno != errno.ESRCH:
                             # If errno is not "no such process", raise
                             raise
 
-                    out = [
-                        'Process took more than {0} seconds to complete. '
-                        'Process Killed!'.format(timeout)
-                    ]
-                    if catch_stderr:
-                        err = ['Process killed, unable to catch stderr output']
-                        if with_retcode:
-                            return out, err, process.returncode
-                        else:
-                            return out, err
-                    if with_retcode:
-                        return out, process.returncode
-                    else:
-                        return out
+                    return format_return(
+                        process.returncode,
+                        *process.communicate(),
+                        timed_out=True
+                    )
 
-                if process.returncode is not None:
-                    break
         tmp_file.seek(0)
 
         if sys.version_info >= (3,):
@@ -355,29 +429,14 @@ class ShellTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                 process.stdout.close()
             if process.stderr is not None:
                 process.stderr.close()
+
             # pylint: disable=maybe-no-member
             try:
-                if with_retcode:
-                    if out is not None and err is not None:
-                        if not raw:
-                            return out.splitlines(), err.splitlines(), process.returncode
-                        else:
-                            return out, err, process.returncode
-                    return out.splitlines(), [], process.returncode
-                else:
-                    if out is not None and err is not None:
-                        if not raw:
-                            return out.splitlines(), err.splitlines()
-                        else:
-                            return out, err
-                    if not raw:
-                        return out.splitlines(), []
-                    else:
-                        return out, []
+                return format_return(process.returncode, out, err or '')
             finally:
                 try:
                     if os.path.exists(tmp_file.name):
-                        if isinstance(tmp_file.name, str):
+                        if isinstance(tmp_file.name, six.string_types):
                             # tmp_file.name is an int when using SpooledTemporaryFiles
                             # int types cannot be used with os.remove() in Python 3
                             os.remove(tmp_file.name)
@@ -396,20 +455,11 @@ class ShellTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
             process.stdout.close()
 
         try:
-            if with_retcode:
-                if not raw:
-                    return out.splitlines(), process.returncode
-                else:
-                    return out, process.returncode
-            else:
-                if not raw:
-                    return out.splitlines()
-                else:
-                    return out
+            return format_return(process.returncode, out)
         finally:
             try:
                 if os.path.exists(tmp_file.name):
-                    if isinstance(tmp_file.name, str):
+                    if isinstance(tmp_file.name, six.string_types):
                         # tmp_file.name is an int when using SpooledTemporaryFiles
                         # int types cannot be used with os.remove() in Python 3
                         os.remove(tmp_file.name)
@@ -441,25 +491,29 @@ class ShellCase(ShellTestCase, AdaptedConfigurationTestCaseMixin, ScriptPathMixi
         '''
         Execute salt
         '''
-        arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
-        return self.run_script('salt',
-                               arg_str,
-                               with_retcode=with_retcode,
-                               catch_stderr=catch_stderr,
-                               timeout=timeout)
+        arg_str = '-c {0} -t {1} {2}'.format(self.get_config_dir(), timeout, arg_str)
+        ret = self.run_script('salt',
+                              arg_str,
+                              with_retcode=with_retcode,
+                              catch_stderr=catch_stderr,
+                              timeout=timeout)
+        log.debug('Result of run_salt for command \'%s\': %s', arg_str, ret)
+        return ret
 
     def run_spm(self, arg_str, with_retcode=False, catch_stderr=False, timeout=60):  # pylint: disable=W0221
         '''
         Execute spm
         '''
-        return self.run_script('spm',
-                               arg_str,
-                               with_retcode=with_retcode,
-                               catch_stderr=catch_stderr,
-                               timeout=timeout)
+        ret = self.run_script('spm',
+                              arg_str,
+                              with_retcode=with_retcode,
+                              catch_stderr=catch_stderr,
+                              timeout=timeout)
+        log.debug('Result of run_spm for command \'%s\': %s', arg_str, ret)
+        return ret
 
-    def run_ssh(self, arg_str, with_retcode=False, catch_stderr=False,
-                timeout=60, wipe=True, raw=False):  # pylint: disable=W0221
+    def run_ssh(self, arg_str, with_retcode=False, catch_stderr=False,  # pylint: disable=W0221
+                timeout=60, wipe=True, raw=False):
         '''
         Execute salt-ssh
         '''
@@ -470,26 +524,32 @@ class ShellCase(ShellTestCase, AdaptedConfigurationTestCaseMixin, ScriptPathMixi
             os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'key_test'),
             os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'roster'),
             arg_str)
-        return self.run_script('salt-ssh',
-                               arg_str,
-                               with_retcode=with_retcode,
-                               catch_stderr=catch_stderr,
-                               timeout=timeout,
-                               raw=True)
+        ret = self.run_script('salt-ssh',
+                              arg_str,
+                              with_retcode=with_retcode,
+                              catch_stderr=catch_stderr,
+                              timeout=timeout,
+                              raw=True)
+        log.debug('Result of run_ssh for command \'%s\': %s', arg_str, ret)
+        return ret
 
-    def run_run(self, arg_str, with_retcode=False, catch_stderr=False, async=False, timeout=60, config_dir=None):
+    def run_run(self, arg_str, with_retcode=False, catch_stderr=False,
+                asynchronous=False, timeout=180, config_dir=None, **kwargs):
         '''
         Execute salt-run
         '''
+        asynchronous = kwargs.get('async', asynchronous)
         arg_str = '-c {0}{async_flag} -t {timeout} {1}'.format(config_dir or self.get_config_dir(),
                                                                arg_str,
                                                                timeout=timeout,
-                                                               async_flag=' --async' if async else '')
-        return self.run_script('salt-run',
-                               arg_str,
-                               with_retcode=with_retcode,
-                               catch_stderr=catch_stderr,
-                               timeout=60)
+                                                               async_flag=' --async' if asynchronous else '')
+        ret = self.run_script('salt-run',
+                              arg_str,
+                              with_retcode=with_retcode,
+                              catch_stderr=catch_stderr,
+                              timeout=60)
+        log.debug('Result of run_run for command \'%s\': %s', arg_str, ret)
+        return ret
 
     def run_run_plus(self, fun, *arg, **kwargs):
         '''
@@ -527,6 +587,8 @@ class ShellCase(ShellTestCase, AdaptedConfigurationTestCaseMixin, ScriptPathMixi
         finally:
             opts['output_file'].close()
 
+        log.debug('Result of run_run_plus for fun \'%s\' with arg \'%s\': %s',
+                  fun, opts_arg, ret)
         return ret
 
     def run_key(self, arg_str, catch_stderr=False, with_retcode=False):
@@ -534,16 +596,20 @@ class ShellCase(ShellTestCase, AdaptedConfigurationTestCaseMixin, ScriptPathMixi
         Execute salt-key
         '''
         arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
-        return self.run_script('salt-key',
-                               arg_str,
-                               catch_stderr=catch_stderr,
-                               with_retcode=with_retcode,
-                               timeout=60)
+        ret = self.run_script('salt-key',
+                              arg_str,
+                              catch_stderr=catch_stderr,
+                              with_retcode=with_retcode,
+                              timeout=60)
+        log.debug('Result of run_key for command \'%s\': %s', arg_str, ret)
+        return ret
 
     def run_cp(self, arg_str, with_retcode=False, catch_stderr=False):
         '''
         Execute salt-cp
         '''
+        # Note: not logging result of run_cp because it will log a bunch of
+        # bytes which will not be very helpful.
         arg_str = '--config-dir {0} {1}'.format(self.get_config_dir(), arg_str)
         return self.run_script('salt-cp',
                                arg_str,
@@ -557,21 +623,25 @@ class ShellCase(ShellTestCase, AdaptedConfigurationTestCaseMixin, ScriptPathMixi
         '''
         arg_str = '{0} --config-dir {1} {2}'.format('--local' if local else '',
                                                     self.get_config_dir(), arg_str)
-        return self.run_script('salt-call',
-                               arg_str,
-                               with_retcode=with_retcode,
-                               catch_stderr=catch_stderr,
-                               timeout=60)
+        ret = self.run_script('salt-call',
+                              arg_str,
+                              with_retcode=with_retcode,
+                              catch_stderr=catch_stderr,
+                              timeout=60)
+        log.debug('Result of run_call for command \'%s\': %s', arg_str, ret)
+        return ret
 
     def run_cloud(self, arg_str, catch_stderr=False, timeout=30):
         '''
         Execute salt-cloud
         '''
         arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
-        return self.run_script('salt-cloud',
-                               arg_str,
-                               catch_stderr,
-                               timeout=timeout)
+        ret = self.run_script('salt-cloud',
+                              arg_str,
+                              catch_stderr,
+                              timeout=timeout)
+        log.debug('Result of run_cloud for command \'%s\': %s', arg_str, ret)
+        return ret
 
 
 class SPMTestUserInterface(object):
@@ -608,16 +678,17 @@ class SPMCase(TestCase, AdaptedConfigurationTestCaseMixin):
         for f_dir in dirs:
             os.makedirs(f_dir)
 
-        import salt.utils
+        # Late import
+        import salt.utils.files
 
-        with salt.utils.fopen(self.formula_sls, 'w') as fp:
+        with salt.utils.files.fopen(self.formula_sls, 'w') as fp:
             fp.write(textwrap.dedent('''\
                      install-apache:
                        pkg.installed:
                          - name: apache2
                      '''))
 
-        with salt.utils.fopen(self.formula_file, 'w') as fp:
+        with salt.utils.files.fopen(self.formula_file, 'w') as fp:
             fp.write(textwrap.dedent('''\
                      name: apache
                      os: RedHat, Debian, Ubuntu, Suse, FreeBSD
@@ -653,14 +724,14 @@ class SPMCase(TestCase, AdaptedConfigurationTestCaseMixin):
             'spm_share_dir': os.path.join(self._tmp_spm, 'share'),
         })
 
-        import salt.utils
-        import yaml
+        import salt.utils.files
+        import salt.utils.yaml
 
         if not os.path.isdir(config['formula_path']):
             os.makedirs(config['formula_path'])
 
-        with salt.utils.fopen(os.path.join(self._tmp_spm, 'spm'), 'w') as fp:
-            fp.write(yaml.dump(config))
+        with salt.utils.files.fopen(os.path.join(self._tmp_spm, 'spm'), 'w') as fp:
+            salt.utils.yaml.safe_dump(config, fp)
 
         return config
 
@@ -674,9 +745,10 @@ class SPMCase(TestCase, AdaptedConfigurationTestCaseMixin):
         repo_conf_dir = self.config['spm_repos_config'] + '.d'
         os.makedirs(repo_conf_dir)
 
-        import salt.utils
+        # Late import
+        import salt.utils.files
 
-        with salt.utils.fopen(os.path.join(repo_conf_dir, 'spm.repo'), 'w') as fp:
+        with salt.utils.files.fopen(os.path.join(repo_conf_dir, 'spm.repo'), 'w') as fp:
             fp.write(textwrap.dedent('''\
                      local_repo:
                        url: file://{0}
@@ -714,7 +786,7 @@ class ModuleCase(TestCase, SaltClientTestCaseMixin):
         behavior of the raw function call
         '''
         know_to_return_none = (
-            'file.chown', 'file.chgrp', 'ssh.recv_known_host'
+            'file.chown', 'file.chgrp', 'ssh.recv_known_host_entries'
         )
         if 'f_arg' in kwargs:
             kwargs['arg'] = kwargs.pop('f_arg')
@@ -816,14 +888,20 @@ class SSHCase(ShellCase):
     def _arg_str(self, function, arg):
         return '{0} {1}'.format(function, ' '.join(arg))
 
-    def run_function(self, function, arg=(), timeout=90, wipe=True, raw=False, **kwargs):
+    def run_function(self, function, arg=(), timeout=180, wipe=True, raw=False, **kwargs):
         '''
-        We use a 90s timeout here, which some slower systems do end up needing
+        We use a 180s timeout here, which some slower systems do end up needing
         '''
         ret = self.run_ssh(self._arg_str(function, arg), timeout=timeout,
                            wipe=wipe, raw=raw)
+        log.debug('SSHCase run_function executed %s with arg %s', function, arg)
+        log.debug('SSHCase JSON return: %s', ret)
+
+        # Late import
+        import salt.utils.json
+
         try:
-            return json.loads(ret)['localhost']
+            return salt.utils.json.loads(ret)['localhost']
         except Exception:
             return ret
 
