@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
-    :codeauthor: :email:`Joe Julian <me@joejulian.name>`
+    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
+    :codeauthor: Joe Julian <me@joejulian.name>
 '''
 
 # Import Python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import Salt Testing Libs
 from tests.support.mixins import LoaderModuleMockMixin
