@@ -14,7 +14,7 @@ from tests.support.unit import TestCase, skipIf
 from tests.support.mock import MagicMock, patch, NO_MOCK, NO_MOCK_REASON
 
 # Import third party libs
-import salt.ext.six as six
+from salt.ext import six
 
 
 @skipIf(NO_MOCK, NO_MOCK_REASON)
@@ -33,7 +33,7 @@ class ParallelsTestCase(TestCase, LoaderModuleMockMixin):
         mock_false = MagicMock(return_value=False)
 
         # Validate false return
-        with patch('salt.utils.which', mock_false):
+        with patch('salt.utils.path.which', mock_false):
             ret = parallels.__virtual__()
             self.assertTrue(isinstance(ret, tuple))
             self.assertEqual(len(ret), 2)
@@ -41,7 +41,7 @@ class ParallelsTestCase(TestCase, LoaderModuleMockMixin):
             self.assertTrue(isinstance(ret[1], six.string_types))
 
         # Validate true return
-        with patch('salt.utils.which', mock_true):
+        with patch('salt.utils.path.which', mock_true):
             ret = parallels.__virtual__()
             self.assertTrue(ret)
             self.assertEqual(ret, 'parallels')
